@@ -22,11 +22,11 @@ pub async fn getast(client: &Client, context: &str) -> Option<DocumentSymbolResp
     }
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(context, None);
-    let tree = thetree.unwrap();
+    let tree = parse.parse(context, None)?;
     getsubast(tree.root_node(), &context.lines().collect(), line > 10000)
         .map(DocumentSymbolResponse::Nested)
 }
+
 #[allow(deprecated)]
 fn getsubast(
     input: tree_sitter::Node,
@@ -53,6 +53,7 @@ fn getsubast(
                 let Some(name) = &source[h][x..y].split(' ').next() else {
                     continue;
                 };
+
                 asts.push(DocumentSymbol {
                     name: name.to_string(),
                     detail: None,
@@ -233,12 +234,38 @@ fn getsubast(
     }
 }
 
-#[test]
-fn test_ast() {
-    let context = include_str!("../assert/base/formatbefore.cmake");
-    let mut parse = tree_sitter::Parser::new();
-    parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(context, None).unwrap();
+#[cfg(test)]
+mod ast_test {
+    use super::*;
+    #[test]
+    fn test_ast_1() {
+        let context = include_str!("../assert/ast_test/bast_test.cmake");
+        let mut parse = tree_sitter::Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let thetree = parse.parse(context, None).unwrap();
 
-    assert!(getsubast(thetree.root_node(), &context.lines().collect(), false).is_some());
+        assert!(getsubast(thetree.root_node(), &context.lines().collect(), false).is_some());
+    }
+
+    #[test]
+    fn test_ast_2() {
+        let context = include_str!("../assert/ast_test/nheko_test.cmake");
+        let mut parse = tree_sitter::Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let thetree = parse.parse(context, None).unwrap();
+
+        assert!(getsubast(thetree.root_node(), &context.lines().collect(), false).is_some());
+    }
+
+    #[test]
+    fn test_ast_3() {
+        let context = r#"
+# Just comment here
+"#;
+        let mut parse = tree_sitter::Parser::new();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+        let thetree = parse.parse(context, None).unwrap();
+
+        assert!(getsubast(thetree.root_node(), &context.lines().collect(), false).is_none());
+    }
 }
