@@ -12,49 +12,40 @@ use crate::{
     languageserver::BUFFERS_CACHE,
     scansubs::TREE_MAP,
     utils::{
-        replace_placeholders,
-        treehelper::{get_position_string, point_to_position},
-        CACHE_CMAKE_PACKAGES_WITHKEYS,
+        gen_module_pattern, get_the_packagename, replace_placeholders,
+        treehelper::{get_point_string, point_to_position, position_to_point},
+        LineCommentTmp, CACHE_CMAKE_PACKAGES_WITHKEYS,
     },
     CMakeNodeKinds,
 };
-use lsp_types::{MessageType, Position, Range, Url};
 use std::sync::LazyLock;
-use tower_lsp::lsp_types;
+use tower_lsp::lsp_types::{self, Location, MessageType, Position, Range, Url};
 mod findpackage;
 mod include;
 mod subdirectory;
 use crate::utils::treehelper::{get_pos_type, PositionType};
-use lsp_types::Location;
 
 use tree_sitter::Node;
 
-pub type JumpKV = HashMap<String, (Location, String)>;
+/// Storage the information when jump
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct JumpCacheUnit {
+    pub location: Location,
+    pub document_info: String,
+}
+
+pub type JumpKV = HashMap<String, JumpCacheUnit>;
 
 pub static JUMP_CACHE: LazyLock<Arc<Mutex<JumpKV>>> =
     LazyLock::new(|| Arc::new(Mutex::new(HashMap::new())));
 
 const JUMP_FILITER_KIND: &[&str] = &["identifier", "unquoted_argument"];
 
-// FIXME: I do not know the way to gen module_pattern on windows
-#[allow(unused_variables)]
-fn gen_module_pattern(subpath: &str) -> Option<String> {
-    #[cfg(unix)]
-    #[cfg(not(target_os = "android"))]
-    {
-        Some(format!("/usr/share/cmake*/Modules/{subpath}.cmake"))
-    }
-    #[cfg(target_os = "android")]
-    {
-        let Ok(prefix) = std::env::var("PREFIX") else {
-            return None;
-        };
-        Some(format!("{prefix}/cmake*/Modules/{subpath}.cmake"))
-    }
-    #[cfg(not(unix))]
-    {
-        None
-    }
+#[derive(Debug, Clone, PartialEq, Eq)]
+struct CacheDataUnit {
+    key: String,
+    location: Location,
+    document_info: String,
 }
 
 pub async fn update_cache<P: AsRef<Path>>(path: P, context: &str) -> Option<()> {
@@ -66,17 +57,26 @@ pub async fn update_cache<P: AsRef<Path>>(path: P, context: &str) -> Option<()>
         tree.root_node(),
         &context.lines().collect(),
         path.as_ref(),
-        PositionType::Variable,
-        None,
+        PositionType::VarOrFun,
         &mut Vec::new(),
         &mut Vec::new(),
         true,
         true,
     )?;
     let mut cache = JUMP_CACHE.lock().await;
-    for (key, position, description) in result_data {
-        *cache.entry(key).or_insert((position, description)) =
-            (position.clone(), description.clone());
+    for CacheDataUnit {
+        key,
+        location,
+        document_info,
+    } in result_data
+    {
+        cache.insert(
+            key,
+            JumpCacheUnit {
+                location,
+                document_info,
+            },
+        );
     }
     None
 }
@@ -87,7 +87,7 @@ pub async fn get_cached_defs<P: AsRef<Path>>(path: P, key: &str) -> Option<Locat
     let tree_map = TREE_MAP.lock().await;
 
     let jump_cache = JUMP_CACHE.lock().await;
-    if let Some((location, _)) = jump_cache.get(key) {
+    if let Some(JumpCacheUnit { location, .. }) = jump_cache.get(key) {
         return Some(location.clone());
     }
     drop(jump_cache);
@@ -100,14 +100,14 @@ pub async fn get_cached_defs<P: AsRef<Path>>(path: P, key: &str) -> Option<Locat
         drop(buffer_cache);
         update_cache(&path, context.as_str()).await;
         let jump_cache = JUMP_CACHE.lock().await;
-        if let Some((location, _)) = jump_cache.get(key) {
+        if let Some(JumpCacheUnit { location, .. }) = jump_cache.get(key) {
             return Some(location.clone());
         }
     }
 
     while let Some(parent) = tree_map.get(&path) {
         let jump_cache = JUMP_CACHE.lock().await;
-        if let Some((location, _)) = jump_cache.get(key) {
+        if let Some(JumpCacheUnit { location, .. }) = jump_cache.get(key) {
             return Some(location.clone());
         }
         drop(jump_cache);
@@ -120,7 +120,7 @@ pub async fn get_cached_defs<P: AsRef<Path>>(path: P, key: &str) -> Option<Locat
             drop(buffer_cache);
             update_cache(&path, context.as_str()).await;
             let jump_cache = JUMP_CACHE.lock().await;
-            if let Some((location, _)) = jump_cache.get(key) {
+            if let Some(JumpCacheUnit { location, .. }) = jump_cache.get(key) {
                 return Some(location.clone());
             }
         }
@@ -129,6 +129,7 @@ pub async fn get_cached_defs<P: AsRef<Path>>(path: P, key: &str) -> Option<Locat
 
     None
 }
+
 /// find the definition
 pub async fn godef(
     location: Position,
@@ -136,61 +137,71 @@ pub async fn godef(
     originuri: &PathBuf,
     client: &tower_lsp::Client,
     is_jump: bool,
+) -> Option<Vec<Location>> {
+    let current_point = position_to_point(location);
+    let locations = godef_inner(current_point, source, originuri, is_jump).await;
+    if locations.is_none() {
+        client
+            .log_message(MessageType::INFO, "Not find any locations")
+            .await;
+    }
+    locations
+}
+
+async fn godef_inner(
+    location: tree_sitter::Point,
+    source: &str,
+    originuri: &PathBuf,
+    is_jump: bool,
 ) -> Option<Vec<Location>> {
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(source, None);
-    let tree = thetree.unwrap();
-    let positionstring = get_position_string(location, tree.root_node(), source);
-    match positionstring {
-        Some(tofind) if (tofind != "(" && tofind != ")") => {
-            let jumptype = get_pos_type(location, tree.root_node(), source, PositionType::Variable);
-            match jumptype {
-                PositionType::Variable => {
-                    let mut locations = vec![];
-                    if let Some(jump_cache) = get_cached_defs(&originuri, tofind.as_str()).await {
-                        if is_jump {
-                            return Some(vec![jump_cache]);
-                        } else {
-                            locations.push(jump_cache);
-                        }
-                    }
+    let tree = parse.parse(source, None)?;
 
-                    let newsource: Vec<&str> = source.lines().collect();
-                    if let Some(mut defdata) =
-                        simplegodefsub(tree.root_node(), &newsource, &tofind, originuri, is_jump)
-                    {
-                        locations.append(&mut defdata);
-                    }
-                    if locations.is_empty() {
-                        None
-                    } else {
-                        Some(locations)
-                    }
-                }
-                PositionType::FindPackage
-                | PositionType::TargetLink
-                | PositionType::TargetInclude => {
-                    let tofind = tofind.split('_').collect::<Vec<&str>>()[0].to_string();
-                    findpackage::cmpfindpackage(tofind, client).await
-                }
-                PositionType::NotFind => None,
-                #[cfg(unix)]
-                PositionType::FindPkgConfig => None,
-                PositionType::Include => {
-                    let fixed_url = replace_placeholders(&tofind)?;
-                    include::cmpinclude(originuri, &fixed_url, client).await
-                }
-                PositionType::SubDir => {
-                    let fixed_url = replace_placeholders(&tofind)?;
-                    subdirectory::cmpsubdirectory(originuri, &fixed_url, client).await
+    let tofind = get_point_string(location, tree.root_node(), &source.lines().collect())?;
+
+    let jumptype = get_pos_type(location, tree.root_node(), source);
+
+    match jumptype {
+        PositionType::VarOrFun => {
+            let mut locations = vec![];
+            if let Some(jump_cache) = get_cached_defs(&originuri, tofind.as_str()).await {
+                if is_jump {
+                    return Some(vec![jump_cache]);
                 }
+                locations.push(jump_cache);
+            }
+
+            let newsource: Vec<&str> = source.lines().collect();
+            if let Some(mut defdata) =
+                simplegodefsub(tree.root_node(), &newsource, &tofind, originuri, is_jump)
+            {
+                locations.append(&mut defdata);
+            }
+            if locations.is_empty() {
+                None
+            } else {
+                Some(locations)
             }
         }
-        None => None,
-        _ => {
-            client.log_message(MessageType::INFO, "Empty").await;
-            None
+        PositionType::FindPackageSpace(space) => {
+            let tofind = format!("{space}{}", get_the_packagename(&tofind));
+            findpackage::cmpfindpackage(&tofind)
+        }
+        PositionType::FindPackage | PositionType::TargetLink | PositionType::TargetInclude => {
+            let tofind = get_the_packagename(&tofind);
+            findpackage::cmpfindpackage(tofind)
+        }
+        PositionType::Unknown | PositionType::Comment | PositionType::ArgumentOrList => None,
+        #[cfg(unix)]
+        PositionType::FindPkgConfig => None,
+        PositionType::Include => {
+            let fixed_url = replace_placeholders(&tofind)?;
+            include::cmpinclude(originuri, &fixed_url)
+        }
+        PositionType::SubDir => {
+            let fixed_url = replace_placeholders(&tofind)?;
+            subdirectory::cmpsubdirectory(originuri, &fixed_url)
         }
     }
 }
@@ -250,29 +261,30 @@ fn getsubdef(
     source: &Vec<&str>,
     local_path: &Path,
     postype: PositionType,
-    location: Option<Position>,
     include_files: &mut Vec<PathBuf>,
     complete_packages: &mut Vec<String>,
     should_in: bool, // if is searched to findpackage, it should not in
     find_cmake_in_package: bool,
-) -> Option<Vec<(String, Location, String)>> {
-    if let Some(location) = location {
-        if input.start_position().row as u32 > location.line {
-            return None;
-        }
-    }
+) -> Option<Vec<CacheDataUnit>> {
     let mut course = input.walk();
-    let mut defs: Vec<(String, Location, String)> = vec![];
+    let mut defs: Vec<CacheDataUnit> = vec![];
+    let mut line_comment_tmp = LineCommentTmp {
+        start_y: 0,
+        comment: "",
+    };
     for child in input.children(&mut course) {
-        if let Some(location) = location {
-            if child.start_position().row as u32 > location.line {
-                // if this child is below row, then break all loop
-                break;
-            }
-        }
         let start = point_to_position(child.start_position());
         let end = point_to_position(child.end_position());
         match child.kind() {
+            CMakeNodeKinds::LINE_COMMENT => {
+                let start_y = child.start_position().row;
+                let start_x = child.start_position().column;
+                let end_x = child.end_position().column;
+                line_comment_tmp = LineCommentTmp {
+                    start_y,
+                    comment: &source[start_y][start_x..end_x],
+                }
+            }
             CMakeNodeKinds::FUNCTION_DEF => {
                 let Some(function_whole) = child.child(0) else {
                     continue;
@@ -289,14 +301,19 @@ fn getsubdef(
                 let Some(name) = &source[h][x..y].split(' ').next() else {
                     continue;
                 };
-                defs.push((
-                    name.to_string(),
-                    Location {
+                let mut document_info = format!("defined function\nfrom: {}", local_path.display());
+
+                if line_comment_tmp.is_node_comment(h) {
+                    document_info = format!("{}\n\n{}", document_info, line_comment_tmp.comment());
+                }
+                defs.push(CacheDataUnit {
+                    key: name.to_string(),
+                    location: Location {
                         uri: Url::from_file_path(local_path).unwrap(),
                         range: Range { start, end },
                     },
-                    format!("function in {}", local_path.display()),
-                ));
+                    document_info,
+                });
             }
             CMakeNodeKinds::MACRO_DEF => {
                 let Some(macro_whole) = child.child(0) else {
@@ -314,14 +331,19 @@ fn getsubdef(
                 let Some(name) = &source[h][x..y].split(' ').next() else {
                     continue;
                 };
-                defs.push((
-                    name.to_string(),
-                    Location {
+                let mut document_info = format!("defined macro\nfrom: {}", local_path.display());
+
+                if line_comment_tmp.is_node_comment(h) {
+                    document_info = format!("{}\n\n{}", document_info, line_comment_tmp.comment());
+                }
+                defs.push(CacheDataUnit {
+                    key: name.to_string(),
+                    location: Location {
                         uri: Url::from_file_path(local_path).unwrap(),
                         range: Range { start, end },
                     },
-                    format!("macro in {}", local_path.display()),
-                ));
+                    document_info,
+                });
             }
             CMakeNodeKinds::IF_CONDITION | CMakeNodeKinds::FOREACH_LOOP | CMakeNodeKinds::BODY => {
                 if let Some(mut message) = getsubdef(
@@ -329,7 +351,6 @@ fn getsubdef(
                     source,
                     local_path,
                     postype,
-                    location,
                     include_files,
                     complete_packages,
                     true,
@@ -375,7 +396,7 @@ fn getsubdef(
                             continue;
                         }
                         if let Ok(true) = subpath.try_exists() {
-                            if let Some(mut comps) = include::scanner_include_def(
+                            if let Some(mut comps) = include::scanner_include_defs(
                                 &subpath,
                                 postype,
                                 include_files,
@@ -467,19 +488,33 @@ fn getsubdef(
                     let h = ids.start_position().row;
                     let x = ids.start_position().column;
                     let y = ids.end_position().column;
-                    let name = &source[h][x..y].split(' ').next();
-
-                    let Some(name) = name.map(|name| name.to_string()) else {
+                    let Some(name) = &source[h][x..y].split(' ').next() else {
                         continue;
                     };
-                    defs.push((
-                        name.to_string(),
-                        Location {
+                    let mut document_info =
+                        format!("defined variable\nfrom: {}", local_path.display());
+
+                    if line_comment_tmp.is_node_comment(h) {
+                        document_info =
+                            format!("{}\n\n{}", document_info, line_comment_tmp.comment());
+                    }
+                    defs.push(CacheDataUnit {
+                        key: name.to_string(),
+                        location: Location {
                             uri: Url::from_file_path(local_path).unwrap(),
-                            range: Range { start, end },
+                            range: Range {
+                                start: Position {
+                                    line: h as u32,
+                                    character: x as u32,
+                                },
+                                end: Position {
+                                    line: h as u32,
+                                    character: y as u32,
+                                },
+                            },
                         },
-                        format!("value in {}", local_path.display()),
-                    ));
+                        document_info,
+                    });
                 }
             }
             CMakeNodeKinds::IDENTIFIER => {
@@ -492,7 +527,6 @@ fn getsubdef(
             source,
             local_path,
             postype,
-            location,
             include_files,
             complete_packages,
             true,
@@ -513,7 +547,7 @@ fn get_cmake_package_defs(
     postype: PositionType,
     include_files: &mut Vec<PathBuf>,
     complete_packages: &mut Vec<String>,
-) -> Option<Vec<(String, Location, String)>> {
+) -> Option<Vec<CacheDataUnit>> {
     let packageinfo = CACHE_CMAKE_PACKAGES_WITHKEYS.get(package_name)?;
     let mut complete_infos = Vec::new();
 
@@ -528,28 +562,200 @@ fn get_cmake_package_defs(
 
     Some(complete_infos)
 }
+#[cfg(test)]
+mod jump_test {
+    use tree_sitter::Point;
+
+    use super::*;
+
+    #[tokio::test]
+    async fn tst_jump_subdir() {
+        use std::fs;
+
+        use std::fs::File;
+        use std::io::Write;
+        use tempfile::tempdir;
+
+        let jump_file_src = r#"add_subdirectory(abcd_test)"#;
+
+        let dir = tempdir().unwrap();
+        let top_cmake = dir.path().join("CMakeLists.txt");
+        let mut top_file = File::create_new(&top_cmake).unwrap();
+        top_file.write(jump_file_src.as_bytes()).unwrap();
+        let subdir = dir.path().join("abcd_test");
+        fs::create_dir_all(&subdir).unwrap();
+        let subdir_file = subdir.join("CMakeLists.txt");
+        File::create_new(&subdir_file).unwrap();
+
+        let locations = godef_inner(
+            Point { row: 0, column: 20 },
+            &jump_file_src,
+            &top_cmake,
+            true,
+        )
+        .await
+        .unwrap();
 
-#[test]
-fn test_module_pattern() {
-    #[cfg(unix)]
-    #[cfg(not(target_os = "android"))]
-    assert_eq!(
-        gen_module_pattern("GNUInstallDirs"),
-        Some("/usr/share/cmake*/Modules/GNUInstallDirs.cmake".to_string())
-    );
-    #[cfg(target_os = "android")]
-    {
-        std::env::set_var("PREFIX", "/data/data/com.termux/files/usr");
         assert_eq!(
-            gen_module_pattern("GNUInstallDirs"),
-            Some(
-                "/data/data/com.termux/files/usr/share/cmake*/Modules/GNUInstallDirs.cmake"
-                    .to_string()
-            )
-        );
+            locations,
+            vec![Location {
+                uri: Url::from_file_path(subdir_file).unwrap(),
+                range: lsp_types::Range {
+                    start: lsp_types::Position {
+                        line: 0,
+                        character: 0,
+                    },
+                    end: lsp_types::Position {
+                        line: 0,
+                        character: 0,
+                    },
+                }
+            }]
+        )
     }
-    #[cfg(not(unix))]
-    {
-        assert_eq!(gen_module_pattern("GNUInstallDirs"), None);
+
+    #[tokio::test]
+    async fn tst_jump_variable() {
+        use std::fs;
+
+        use std::fs::File;
+        use std::io::Write;
+        use tempfile::tempdir;
+
+        let jump_file_src = r#"
+set(ABCD 1234)
+message(INFO "${ABCD}")
+set(ROOT_DIR "ABCD" STRING CACHE "ROOTDIR")
+include("${ROOT_DIR}/abcd_test")
+add_subdirectory(abcd_test)
+"#;
+
+        let dir = tempdir().unwrap();
+        let top_cmake = dir.path().join("CMakeLists.txt");
+        let mut top_file = File::create_new(&top_cmake).unwrap();
+        top_file.write(jump_file_src.as_bytes()).unwrap();
+        let subdir = dir.path().join("abcd_test");
+        fs::create_dir_all(&subdir).unwrap();
+        let subdir_file = subdir.join("CMakeLists.txt");
+        File::create_new(&subdir_file).unwrap();
+
+        let locations = godef_inner(
+            Point { row: 2, column: 18 },
+            &jump_file_src,
+            &top_cmake,
+            true,
+        )
+        .await
+        .unwrap();
+
+        assert_eq!(
+            locations,
+            vec![Location {
+                uri: Url::from_file_path(&top_cmake).unwrap(),
+                range: lsp_types::Range {
+                    start: lsp_types::Position {
+                        line: 1,
+                        character: 4,
+                    },
+                    end: lsp_types::Position {
+                        line: 1,
+                        character: 8,
+                    },
+                }
+            }]
+        );
+        let locations_2 = godef_inner(
+            Point { row: 4, column: 13 },
+            &jump_file_src,
+            &top_cmake,
+            true,
+        )
+        .await
+        .unwrap();
+
+        assert_eq!(
+            locations_2,
+            vec![Location {
+                uri: Url::from_file_path(top_cmake).unwrap(),
+                range: lsp_types::Range {
+                    start: lsp_types::Position {
+                        line: 3,
+                        character: 4,
+                    },
+                    end: lsp_types::Position {
+                        line: 3,
+                        character: 12,
+                    },
+                }
+            }]
+        )
     }
 }
+
+#[test]
+fn test_sub_def() {
+    use std::fs::File;
+    use std::io::Write;
+    use tempfile::tempdir;
+    let dir = tempdir().unwrap();
+    let top_cmake_path = dir.path().join("CMakeLists.txt");
+
+    let mut cmake_file = File::create_new(&top_cmake_path).unwrap();
+    let top_cmake_context = r#"
+include(abcd_test.cmake)
+"#;
+    writeln!(cmake_file, "{}", top_cmake_context).unwrap();
+    let include_cmake_path = dir.path().join("abcd_test.cmake");
+    let mut include_cmake = File::create_new(&include_cmake_path).unwrap();
+    let include_cmake_context = r#"
+set(ABCD "abcd")
+include(efg_test.cmake)
+"#;
+    writeln!(include_cmake, "{}", include_cmake_context).unwrap();
+
+    // NOTE: this is used to test if the include cache append will work
+    let include_cmake_path_2 = dir.path().join("efg_test.cmake");
+    File::create(&include_cmake_path_2).unwrap();
+
+    let mut parse = tree_sitter::Parser::new();
+    parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
+    let thetree = parse.parse(top_cmake_context, None).unwrap();
+
+    let mut include_files = vec![];
+    let data = getsubdef(
+        thetree.root_node(),
+        &top_cmake_context.lines().collect(),
+        &top_cmake_path,
+        PositionType::VarOrFun,
+        &mut include_files,
+        &mut vec![],
+        true,
+        false,
+    )
+    .unwrap();
+
+    assert_eq!(
+        data,
+        vec![CacheDataUnit {
+            key: "ABCD".to_string(),
+            location: Location {
+                uri: Url::from_file_path(&include_cmake_path).unwrap(),
+                range: lsp_types::Range {
+                    start: lsp_types::Position {
+                        line: 1,
+                        character: 4
+                    },
+                    end: lsp_types::Position {
+                        line: 1,
+                        character: 8
+                    }
+                }
+            },
+            document_info: format!("defined variable\nfrom: {}", include_cmake_path.display())
+        }]
+    );
+    assert_eq!(
+        include_files,
+        vec![include_cmake_path_2, include_cmake_path]
+    );
+}
