@@ -7,11 +7,13 @@ use super::Backend;
 use crate::ast;
 use crate::complete;
 use crate::consts::TREESITTER_CMAKE_LANGUAGE;
+use crate::document_link;
 use crate::fileapi;
 use crate::fileapi::DEFAULT_QUERY;
 use crate::filewatcher;
 use crate::formatting::getformat;
 use crate::gammar::checkerror;
+use crate::gammar::ErrorInformation;
 use crate::gammar::LintConfigInfo;
 use crate::hover;
 use crate::jump;
@@ -24,7 +26,6 @@ use crate::utils::treehelper;
 use crate::utils::VCPKG_LIBS;
 use crate::utils::VCPKG_PREFIX;
 use std::collections::HashMap;
-use std::path::Path;
 use std::sync::Arc;
 use std::sync::RwLock;
 use tokio::sync::Mutex;
@@ -76,18 +77,32 @@ impl Backend {
         if !self.path_in_project(uri.path()).await {
             return;
         }
-        let mut parse = Parser::new();
-        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-        let thetree = parse.parse(&context, None);
-        let Some(tree) = thetree else {
+
+        let Ok(file_path) = uri.to_file_path() else {
+            tracing::error!("Cannot transport {uri:?} to file_path");
+            self.client
+                .log_message(
+                    MessageType::ERROR,
+                    format!("Cannot transport {uri:?} to file_path"),
+                )
+                .await;
             return;
         };
-        let gammererror = checkerror(Path::new(uri.path()), &context, tree.root_node(), lint_info);
+
+        let gammererror = checkerror(&file_path, &context, lint_info);
         if let Some(diagnoses) = gammererror {
             let mut pusheddiagnoses = vec![];
-            for (start, end, message, severity) in diagnoses.inner {
-                let pointx = lsp_types::Position::new(start.row as u32, start.column as u32);
-                let pointy = lsp_types::Position::new(end.row as u32, end.column as u32);
+            for ErrorInformation {
+                start_point,
+                end_point,
+                message,
+                severity,
+            } in diagnoses.inner
+            {
+                let pointx =
+                    lsp_types::Position::new(start_point.row as u32, start_point.column as u32);
+                let pointy =
+                    lsp_types::Position::new(end_point.row as u32, end_point.column as u32);
                 let range = Range {
                     start: pointx,
                     end: pointy,
@@ -312,6 +327,13 @@ impl LanguageServer for Backend {
                     None
                 },
                 references_provider: Some(OneOf::Left(true)),
+
+                document_link_provider: Some(DocumentLinkOptions {
+                    resolve_provider: Some(true),
+                    work_done_progress_options: WorkDoneProgressOptions {
+                        work_done_progress: Some(false),
+                    },
+                }),
                 ..ServerCapabilities::default()
             },
         })
@@ -523,12 +545,14 @@ impl LanguageServer for Backend {
         } else {
             1
         };
+        let insert_final_newline = input.options.insert_final_newline.unwrap_or(false);
         match storemap.get(&uri) {
             Some(context) => Ok(getformat(
                 context,
                 &self.client,
                 space_line,
                 input.options.insert_spaces,
+                insert_final_newline,
             )
             .await),
             None => Ok(None),
@@ -652,4 +676,20 @@ impl LanguageServer for Backend {
             None => Ok(None),
         }
     }
+
+    async fn document_link(&self, input: DocumentLinkParams) -> Result<Option<Vec<DocumentLink>>> {
+        let uri = input.text_document.uri;
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
+            }
+        };
+        let storemap = BUFFERS_CACHE.lock().await;
+        let Some(context) = storemap.get(&uri) else {
+            return Ok(None);
+        };
+        Ok(document_link::document_link_search(context, file_path))
+    }
 }
